//
// Wifi mit CC3000, Webclient, Aufruf von ext. PHP-Datei auf Webserver
// Datei: buch3_kap10_cc3000_webclient.ino
// Datum: 09.02.14


#include <Adafruit_CC3000.h>
#include <ccspi.h>
#include <SPI.h>
#include <string.h>
#include "utility/debug.h"
#include<stdlib.h>

// CC3000 Pins
#define ADAFRUIT_CC3000_IRQ   3
#define ADAFRUIT_CC3000_VBAT  5
#define ADAFRUIT_CC3000_CS    10

// CC3000 Objekt
Adafruit_CC3000 cc3000 = Adafruit_CC3000(ADAFRUIT_CC3000_CS, ADAFRUIT_CC3000_IRQ, ADAFRUIT_CC3000_VBAT, SPI_CLOCK_DIV2); 
                                         

// WLAN Parameter
#define WLAN_SSID       "MeinWLAN"
#define WLAN_PASS       "MeinWLANPW"

// Security can be WLAN_SEC_UNSEC, WLAN_SEC_WEP, WLAN_SEC_WPA or WLAN_SEC_WPA2
#define WLAN_SECURITY   WLAN_SEC_WPA2

// Website Parameter
#define WEBSITE  "arduino-praxis.ch"

uint32_t ip;

// Temp
float tempC;
float valTemp;

//LM35 Tempsensor
int TempPin=0;

void setup()
{
  // Initialisierung serielle Schnittstelle
  Serial.begin(115200);
  
  Serial.println(F("\nInitializing..."));
  if (!cc3000.begin())
  {
    Serial.println(F("Couldn't begin()! Check your wiring?"));
    while(1);
  }
 
}

void loop()
{
  // Mit WiFi Netzwerk verbinden
  cc3000.connectToAP(WLAN_SSID, WLAN_PASS, WLAN_SECURITY);
  Serial.println(F("Connected!"));
  
  // DHCP 
  Serial.println(F("Request DHCP"));
  while (!cc3000.checkDHCP())
  {
    delay(100);
  }  

  // IP von Website
  ip = 0;
  Serial.print(WEBSITE);
  Serial.print(F(" -> "));
  while (ip == 0) {
    if (! cc3000.getHostByName(WEBSITE, &ip)) {
      Serial.println(F("Couldn't resolve!"));
    }
    delay(500);
  }
  cc3000.printIPdotsRev(ip);
  
  // Tempsensor einlesen
  valTemp = analogRead(TempPin);
  tempC=(5.0 * valTemp * 100.0)/1024.0;

  // Request senden
    Adafruit_CC3000_Client client = cc3000.connectTCP(ip, 80);
  if (client.connected())
  {
    Serial.println("Connected!");
    client.print("GET /arduinobuch/webclient/temperatur.php?t=");
    client.print(tempC);
    client.println(" HTTP/1.0");
    client.println("HOST:arduino-praxis.ch\n\n");
    client.println();
    client.println();
  }
  else
  {
    Serial.println(F("Connection failed"));    
    return;
  }
  
  Serial.println(F("-------------------------------------"));
  while (client.connected()) {
    while (client.available()) {
      char c = client.read();
      Serial.print(c);
    }
  }
  client.close();
  Serial.println(F("-------------------------------------"));
  
  Serial.println(F("\n\nDisconnecting"));
  cc3000.disconnect();
  
  // 10 Sekunden warten
  delay(10000);
  
}

