//
// Projekt Netzteil
// - Gesamte Funktion (Signalgeber, EEPROM, Keypad), Sollwert via PWM
// Datei: buch3_kap6_projekt_netzteil.ino

#include <Keypad.h>
#include <EEPROM.h>
#include <Wire.h>


const byte ROWS = 4; // 4 Zeilen
const byte COLS = 3; // 3 Spalten
// Aufbau Keypad
char keys[ROWS][COLS] = {
  {'1','2','3'},
  {'4','5','6'},
  {'7','8','9'},
  {'*','0','#'}
};

// Zeilen ROW0, ROW1, ROW2 und ROW3 an Arduino-Pins
byte rowPins[ROWS] = { 10, 9, 8, 7 };
// Spalten COL0, COL1 und COL2 an Arduino Pins
byte colPins[COLS] = { 6, 5, 4 };  
  

// Keypad Objekt
Keypad kpd = Keypad( makeKeymap(keys), rowPins, colPins, ROWS, COLS );
// Variablen Keypad
// Anzahl Zeichen für Eingabe = 5
// Bsp. 1000# = Eingabe 10.00 Volt
char keyreihe[5];
char key;
int count=0;

// EEPROM
int eeaddress = 0;
byte eevalue;

// Sollwert
int varSoll;
byte uv, umv;
// Referenzspannung
float vref=5.00;
// max. Ausgangsspannung
float uaus=15.00;
float teiler;

// Pin PWM Sollwert U
int PWMU=11;

void setup()
{
  // Serielle Schnittstelle Start
  Serial.begin(9600);
  // Sollwert 0
  analogWrite(PWMU, 0);
  delay(500);
  // EEPROM Wert lesen
  eevalue = EEPROM.read(eeaddress);
  varSoll=eevalue;
  // Ausgabe Sollwert
  analogWrite(PWMU, varSoll);
}

void loop()
{
  key = kpd.getKey();
   
  if(key)  // gleichbedeutend mit if(key != NO_KEY)
  {
    switch (key)
    {
      case '*':
        //
        break;
      case '#':
        showKey();
        count=0;
        // Werte Löschen
        for (int zz=0; zz<4; zz++)
        {
          keyreihe[zz]=0;
        }  
        // EEPROM Wert lesen
        eevalue = EEPROM.read(eeaddress);
        varSoll=eevalue;
        break;
      default:
        keyreihe[count]=key;
        count=count+1;
        if (count == 4){
          count=0;
        }
     
    }
   }
  
  // Ausgabe des empfangenen Startzeichens
  Serial.print("Sollwert: ");
  Serial.println(varSoll);
  
  // Sollwert U
  analogWrite(PWMU, varSoll);
}

// 
// Funktionen
//

void showKey()
{
  Serial.print("KP Wrt: ");
  Serial.println(keyreihe);
  Serial.print("KP Int: ");
  int keyreiheint=atoi(keyreihe);
  Serial.println(keyreiheint);
  float usoll = keyreiheint;
  Serial.print("KP Usoll: ");
  Serial.println(usoll);

  // Sollspannung in Volt umrechnen
  usoll=usoll/100;
  // Teilerwert für Sollwert berechnen
  // Bsp. max. Ausgangsspannung 15 V -> 15/vref=15/5=3
  teiler=uaus/vref;
  usoll=usoll/teiler;
  
  // Berechnung Intwert für Ausgabe
  int varSollU = (usoll*255)/vref;
 
  // Wert in EEPROM schreiben
  EEPROM.write(eeaddress, varSollU);
}

